Package: quantro
Title: A test for when to use quantile normalization
Version: 1.44.0
Authors@R: c(
    person("Stephanie", "Hicks", role = c("aut", "cre"), 
            email = "shicks19@jhu.edu", 
            comment = c(ORCID = "0000-0002-7858-0231")),
    person("Rafael", "Irizarry", role = "aut", 
            comment = c(ORCID = "0000-0002-3944-4309")))
Imports: Biobase, minfi, doParallel, foreach, iterators, ggplot2,
        methods, RColorBrewer
Depends: R (>= 4.0)
Suggests: rmarkdown, knitr, RUnit, BiocGenerics, BiocStyle
VignetteBuilder: knitr
RoxygenNote: 6.0.1
Encoding: UTF-8
Description: A data-driven test for the assumptions of
    quantile normalization using raw data such as objects
    that inherit eSets (e.g. ExpressionSet, MethylSet).
    Group level information about each sample (such as
    Tumor / Normal status) must also be provided because
    the test assesses if there are global differences in
    the distributions between the user-defined groups.
License: GPL-3
biocViews: Normalization, Preprocessing, MultipleComparison,
        Microarray, Sequencing
git_url: https://git.bioconductor.org/packages/quantro
git_branch: RELEASE_3_22
git_last_commit: d148fc5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:54:55 UTC; biocbuild
Author: Stephanie Hicks [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7858-0231>),
  Rafael Irizarry [aut] (ORCID: <https://orcid.org/0000-0002-3944-4309>)
Maintainer: Stephanie Hicks <shicks19@jhu.edu>

# Generated by roxygen2: do not edit by hand

export(MSbetween)
export(MSwithin)
export(anova)
export(matboxplot)
export(matdensity)
export(quantro)
export(quantroPlot)
export(quantroPvalPerm)
export(quantroStat)
export(quantroStatPerm)
export(summary)
exportClasses(quantro)
exportMethods(MSbetween)
exportMethods(MSwithin)
exportMethods(anova)
exportMethods(quantroPvalPerm)
exportMethods(quantroStat)
exportMethods(quantroStatPerm)
exportMethods(summary)
import(Biobase)
import(RColorBrewer)
import(doParallel)
import(foreach)
import(ggplot2)
import(iterators)
import(minfi)
importFrom(methods,show)

# quantro

### Why use quantro?

Quantile normalization is one of the most widely used multi-sample normalization tools for the analysis of noisy high-throughput data. Although it was originally developed for gene expression microarrays it is now used across many different high-throughput applications including RNAseq and ChIPseq. However, quantile normalization relies on assumptions about the data generation process that are not appropriate in some context. Unfortunately, no method exists to check for the appropriateness of these assumptions. 

For example in gene expression, we assume that observed differences between the distributions of each sample are due to only technical variation unrelated to biological variation. To normalize the samples, the distributions are forced to be the same. In general, this assumption is justified as only a minority of genes are expected to be differentially expressed between samples, but if the samples are expected to have a high percentage of global differences, it may not be appropriate to use quantile normalization as it may remove interesting global biological variation. 

The **quantro** R-package can be used to test a priori to the data analysis whether global normalization methods such as quantile normalization should be applied. Our method uses the raw unprocessed high-throughput data to test for global differences in the distributions across a set of groups. 

For help with the **quantro** R-package, there is a vignette available in the /vignettes folder.
  
# Installation

The R-package **quantro** can be [installed from the Bioconductor](http://www.bioconductor.org/packages/release/bioc/html/quantro.html)
```s
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("quantro")
```

After installation, the package can be loaded into R.
```s
library(quantro)
```

# Using quantro

The main function in the **quantro** package is `quantro()`.  The `quantro()` function needs two objects: (1) a data frame containing the samples to test for differences between their distributions with observations (rows) and samples (columns) (e.g. let's call it `mySamps`) and (2) a group level factor called `groupFactor` (let's call it `outcome`). This order of this factor variable must match the order of the columns in the `mySamps` object because it contains information about which group each sample is from.

To run the `quantro()` function, 
```
qtest <- quantro(object = mySamps, groupFactor = outcome)
qtest
```
Individual slots can be extracted using accessor methods:
```
summary(qtest)
quantroStat(qtest)
```

A permutation test is performed to assess the statistical significance of the test statistic `quantroStat` from `quantro()`. 

#### Elements in the output from `quantro()` include: 

Element | Description
--------|------------
`summary` | A list that contains (1) number of groups (`nGroups`), (2) total number of samples (`nTotSamples`) (3) number of samples in each group (`nSamplesinGroups`)
`anova` | ANOVA to test if the average medians of the distributions are different across groups
`MSbetween` | mean squared error between groups
`MSwithin` | mean squared error within groups
`quantroStat` | test statistic which is a ratio of the mean squared error between groups of distributions to the mean squared error within groups of distributions
`quantroStatPerm` | If `B` is not equal to 0, then a permutation test was performed to assess the statistical significance of `quantroStat`. These are the test statistics resulting from the permuted samples
`quantroPvalPerm` | If `B` is not equal to 0, then this is the $p$-value associated with the proportion of times the test statistics (`quantroStatPerm`) resulting from the permuted samples were larger than `quantroStat`



# Visualizing the results from the permutation test
There is a second function in the package called `quantroPlot()` which will plot the results from the permutation testing. The plot is a histogram of the  test statistics `quantroStatPerm` from the permuted samples from `quantro()` and the red line is the observed test statistic `quantroStat` from `quantro()`. 


```{r}
qtest <- quantro(object = mySamps, groupFactor = outcome)
quantroPlot(qtest)
```


Additional options in the `quantroPlot()` function include:

Element | Description
--------|-------------
xLab | the x-axis label
yLab | the y-axis label
mainLab | title of the histogram
binWidth | change the binwidth


# Bug reports
Report bugs as issues on the [GitHub repository](https://github.com/stephaniehicks/quantro)


# Contributors

* [Stephanie Hicks](https://github.com/stephaniehicks)
* [Rafael Irizarry](https://github.com/ririzarr)

#' @name flowSorted
#' @title A subset of FlowSorted.DLPFC.450k data set
#'
#' @description 
#' This is the script used to create a subset of the 
#' FlowSorted.DLPFC.450k data set from Bioconductor. 
#' The purpose is to create an example object for the man
#' pages and vignette in this package. 
#' 
#' The object was created using the script in /inst and
#' located in the /data folder. 
#' 
#' @docType data
#' @format A MethylSet object with 1e4 rows (probes) 
#' and 58 columns (samples).
#' 
#' @rdname flowSorted
#' 
NULL

#' @title Density plots of columns in a matrix
#' 
#' @description Plots the density of the columns of a matrix
#' 
#' @param object object an object which is inherited from an 
#' \code{eSet} such as an \code{ExpressionSet} or 
#' \code{MethylSet} object. The \code{object} can also be a 
#' data frame or matrix with observations
#' (e.g. probes or genes) on the rows and samples as the 
#' columns.  
#' @param groupFactor an optional factor variable representing which 
#' group each column in \code{object} belongs to. It is important 
#' that values in \code{groupFactor} be in the same 
#' order as the columns in \code{object}. 
#' @param type the type of lines to plot. Default type is line ("l"). 
#' @param brewer.n the number of colors in the palette from the RColorBrewer 
#' package. Default is 8. 
#' @param brewer.name the name of the palette from the RColorBrewer package. 
#' Default is "Dark2". 
#' @param lty the line type. Default is the solid line. 
#' @param ... other arguments that can be passed to the 
#' code{matplot} function. 
#' 
#' @import RColorBrewer
#' 
#' @return A density plot for each column in \code{object}
#' 
#' @export
#' @examples
#' library(minfi)
#' data(flowSorted)
#' p <- getBeta(flowSorted, offset = 100)
#' pd <- pData(flowSorted)
#' matdensity(object = p, groupFactor = pd$CellType, xlab = "beta values",
#' ylab = "density")
matdensity <- function(object, groupFactor = NULL, type = "l", 
                       lty = 1, brewer.n = 8, brewer.name = "Dark2", ...){
   
    if(inherits(object, "eSet")){
        if(is(object, "ExpressionSet")){ object <- exprs(object) }
        if(is(object, "MethylSet")){ object <- getBeta(object, offset = 100) }
    }
    
    getdensity  <- function(object){
        min.object <- min(object[is.finite(object)], na.rm = TRUE)
        max.object <- max(object[is.finite(object)], na.rm = TRUE)
        densityMat <- apply(object, 2, function(z){
            stats::density(z, from = min.object, to = max.object, na.rm = TRUE)$y
        })
        x = seq(from=min.object, to=max.object, length.out = nrow(densityMat))
        list(densityMat = densityMat, x = x)
    }
    output <- getdensity(object)

    grDevices::palette(brewer.pal(brewer.n, brewer.name))
    col = brewer.pal(brewer.n, brewer.name)
    if(!is.null(groupFactor)){
        groupFactor <- as.factor(groupFactor)
        col = col[as.integer(groupFactor)]
    } else { 
        col = rep(col, ncol(object))
    }

    graphics::matplot(x = output$x, output$densityMat, col = col, type = type, 
            lty = lty, ...)
}

#' @title Box plots of columns in a matrix
#' 
#' @description Box plots of the columns of a matrix, but 
#' the columns are ordered and colored by a group-level 
#' variable 
#' 
#' @param object object an object which is inherited from an 
#' \code{eSet} such as an \code{ExpressionSet} or 
#' \code{MethylSet} object. The \code{object} can also be a 
#' data frame or matrix with observations
#' (e.g. probes or genes) on the rows and samples as the 
#' columns.  
#' @param groupFactor a factor variable representing which 
#' group each column in \code{object} belongs to. It is important 
#' that values in \code{groupFactor} be in the same 
#' order as the columns in \code{object}.
#' @param brewer.n the number of colors in the palette from the RColorBrewer 
#' package. Default is 8. 
#' @param brewer.name the name of the palette from the RColorBrewer package. 
#' Default is "Dark2". 
#' @param las a numeric in (0, 1, 2, 3) to orient the axis labels. 
#' Default is 3 (always vertical). 
#' @param ... other arguments that can be passed to the 
#' code{boxplot} function. 
#' 
#' @return A box plot for each column in \code{object}
#' 
#' @export
#' @examples
#' library(minfi)
#' data(flowSorted)
#' 
#' p <- getBeta(flowSorted, offset = 100)
#' pd <- pData(flowSorted)
#' matboxplot(object = p, groupFactor = pd$CellType)
matboxplot <- function (object, groupFactor, las = 3, 
                        brewer.n = 8, brewer.name = "Dark2", ...){
    
    if(inherits(object, "eSet")){
        if(is(object, "ExpressionSet")){ object <- exprs(object) }
        if(is(object, "MethylSet")){ object <- getBeta(object, offset = 100) }
    }

    groupFactor <- as.factor(as.character(groupFactor))
    objectordered <- object[, order(groupFactor)]
    groupFactorOrdered <- groupFactor[order(groupFactor)]

    grDevices::palette(brewer.pal(brewer.n, brewer.name))
    col = brewer.pal(brewer.n, brewer.name) 
    col = col[as.integer(groupFactorOrdered)]
        
    graphics::boxplot(objectordered, las = las, col = col, ...)
}


#' @title Plot results from \code{quantro} function.
#' 
#' @description This function plots the histogram of the
#' null test statistics from permutation test in the \code{quantro}
#' function. 
#' 
#' @param object a quantro object from \code{quantro} 
#' @param savePlot a TRUE/FALSE object argument
#' determining if the plot will be saved for further use or
#' immediately displayed on the screen.
#' @param xLab label for x-axis
#' @param yLab label for y-axis
#' @param mainLab title of plot
#' @param binWidth binwidth or histogram. Default is the stat_bin default. 
#' 
#' @return A histogram will be plotted containing the null 
#' test statistics when using boostrapped samples. 
#' The red line is the observed test statistic
#' \code{quantroStat} from \code{quantro()}.
#' 
#' @import ggplot2 
#' 
#' @export
#' @examples
#' \donttest{
#' library(minfi)
#' data(flowSorted)
#' p <- getBeta(flowSorted, offset = 100)
#' pd <- pData(flowSorted)
#' 
#' library(doParallel)
#' registerDoParallel(cores=4)
#' qtest <- quantro(p, pd$CellType, B = 100)
#' quantroPlot(qtest)
#' }
quantroPlot <- function(object, savePlot = FALSE, xLab = NULL, 
                        yLab = NULL, mainLab = NULL, binWidth = NULL) 
{
    B <- object@B
    quantroStat <- quantroStat(object)
    quantroStatPerm <- quantroStatPerm(object)
    quantroPvalPerm <- quantroPvalPerm(object)

    if(B == 0){
        stop("quantro was run without any permutation testing. Must define 
             B > 0 in quantro() to plot distribution of test 
             statistics from permutation testing. ")
    }  

    if(is.null(xLab)){ 
        xLab <- paste0("Histogram of test statistics \n from permutation 
                       test with B = ", B)
    }

    if(is.null(yLab)){ yLab <- "density" }

    newPvalPerm <- ifelse(quantroPvalPerm(object) < (1/object@B), 
           paste0("< ", 1 / object@B), round(quantroPvalPerm(object),3))
    if(is.null(mainLab)){ mainLab <- paste0(
        "quantroStat = ", round(quantroStat, 3), 
        ", quantroPvalPerm = ", newPvalPerm)
    }

    value = NULL
    dat = data.frame("value" = quantroStatPerm)
    gmat <- ggplot(dat, aes(x = value)) + 
                geom_histogram(binwidth = binWidth) + 
                geom_vline(xintercept = quantroStat, colour = "red") +
                xlab(xLab) + ylab(yLab) + labs(title = mainLab)

    if (savePlot) { 
        suppressMessages(gmat)
    } else { 
        suppressMessages(print(gmat)) 
    }
}

#' @title Accessors for the 'summary' slot of a quantro object.
#' 
#' @description Accessors for the 'summary' slot of a quantro object.
#' 
#' @usage
#' \S4method{summary}{quantro}(object)
#'
#' @docType methods
#' @name summary
#' @rdname summary
#' @aliases summary summary,quantro-method
#' @param object a \code{quantro} object
#' @param ... other 
#' 
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' summary(qtest)
summary.quantro <- function(object) object@summary

#' @rdname summary
#' @export
setMethod("summary", signature(object="quantro"), summary.quantro)



#' Accessors for the 'anova' slot of a quantro object.
#' 
#' Accessors for the 'anova' slot of a quantro object.
#' 
#' @usage
#' \S4method{anova}{quantro}(object)
#'
#' @docType methods
#' @name anova
#' @rdname anova
#' @aliases anova anova,quantro-method
#' @param object a \code{quantro} object
#' @param ... other
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' anova(qtest)
anova.quantro <- function(object) object@anova

#' @rdname anova
#' @export
setMethod("anova", signature(object="quantro"), anova.quantro)


#' Accessors for the 'MSbetween' slot of a quantro object.
#' 
#' Accessors for the 'MSbetween' slot of a quantro object.
#' 
#' @usage
#' \S4method{MSbetween}{quantro}(object)
#'
#' @docType methods
#' @name MSbetween
#' @rdname MSbetween
#' @aliases MSbetween MSbetween,quantro-method
#' @param object a \code{quantro} object
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' MSbetween(qtest)
MSbetween.quantro <- function(object) object@MSbetween

#' @rdname MSbetween
#' @export
setMethod("MSbetween", signature(object="quantro"), MSbetween.quantro)


#' Accessors for the 'MSwithin' slot of a quantro object.
#' 
#' Accessors for the 'MSwithin' slot of a quantro object.
#' 
#' @usage
#' \S4method{MSwithin}{quantro}(object)
#'
#' @docType methods
#' @name MSwithin
#' @rdname MSwithin
#' @aliases MSwithin MSwithin,quantro-method
#' @param object a \code{quantro} object
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' MSwithin(qtest)
MSwithin.quantro <- function(object) object@MSwithin

#' @rdname MSwithin
#' @export
setMethod("MSwithin", signature(object="quantro"), MSwithin.quantro)




#' Accessors for the 'quantroStat' slot of a quantro object.
#' 
#' Accessors for the 'quantroStat' slot of a quantro object.
#' 
#' @usage
#' \S4method{quantroStat}{quantro}(object)
#'
#' @docType methods
#' @name quantroStat
#' @rdname quantroStat
#' @aliases quantroStat quantroStat,quantro-method
#' @param object a \code{quantro} object
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' quantroStat(qtest)
quantroStat.quantro <- function(object) object@quantroStat

#' @rdname quantroStat
#' @export
setMethod("quantroStat", signature(object="quantro"), quantroStat.quantro)


#' Accessors for the 'quantroStatPerm' slot of a quantro object.
#' 
#' Accessors for the 'quantroStatPerm' slot of a quantro object.
#' 
#' @usage
#' \S4method{quantroStatPerm}{quantro}(object)
#'
#' @docType methods
#' @name quantroStatPerm
#' @rdname quantroStatPerm
#' @aliases quantroStatPerm quantroStatPerm,quantro-method
#' @param object a \code{quantro} object
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' quantroStatPerm(qtest)
quantroStatPerm.quantro <- function(object) object@quantroStatPerm

#' @rdname quantroStatPerm
#' @export
setMethod("quantroStatPerm", signature(object="quantro"), 
          quantroStatPerm.quantro)


#' Accessors for the 'quantroPvalPerm' slot of a quantro object.
#' 
#' Accessors for the 'quantroPvalPerm' slot of a quantro object.
#' 
#' @usage
#' \S4method{quantroPvalPerm}{quantro}(object)
#'
#' @docType methods
#' @name quantroPvalPerm
#' @rdname quantroPvalPerm
#' @aliases quantroPvalPerm quantroPvalPerm,quantro-method
#' @param object a \code{quantro} object
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' pd <- pData(flowSorted)
#' qtest <- quantro(flowSorted, groupFactor = pd$CellType)
#' quantroPvalPerm(qtest)
quantroPvalPerm.quantro <- function(object) object@quantroPvalPerm

#' @rdname quantroPvalPerm
#' @export
setMethod("quantroPvalPerm", signature(object="quantro"), 
          quantroPvalPerm.quantro)

# A class for either numeric or logical
setClassUnion("numericORlogical", c("numeric", "logical"))

#' @title quantro
#' @description This is the S4 class quantro container
#'
#' @exportClass quantro 
#' 
setClass(Class = "quantro", 
         representation = representation(
             summary = "list", B = "numeric",
             anova = "data.frame", MSbetween = "numeric",
             MSwithin = "numeric", quantroStat = "numeric",
             quantroStatPerm = "numericORlogical", 
             quantroPvalPerm = "numericORlogical")
)

setMethod("show", "quantro", 
          function(object){
            cat("quantro: Test for global differences in distributions\n")
            cat("   nGroups: ", paste(object@summary$nGroups), "\n")
            cat("   nTotSamples: ", paste(object@summary$nTotSamples), "\n")
            cat("   nSamplesinGroups: ", 
                paste(object@summary$nSamplesinGroups), "\n")
            cat("   anovaPval: ", paste(round(object@anova$`Pr(>F)`[1],5)), 
                "\n") 
            cat("   quantroStat: ", paste(round(object@quantroStat,5)), "\n")
            if(is.na(object@quantroPvalPerm)){
                noPermutation <- "Use B > 0 for permutation testing."
                cat("   quantroPvalPerm: ", paste(noPermutation), "\n")
            } else {
                showPval <- ifelse(object@quantroPvalPerm < (1/object@B), 
                       paste0("< ", 1 / object@B), object@quantroPvalPerm)
                cat("   quantroPvalPerm: ", paste(showPval), "\n")  
            }
          }
)

#' @title quantro
#'
#' @description This is a function that tests for global differences between 
#' groups of distributions which asses whether global normalization 
#' methods such as quantile normalization should be applied. 
#' This function defines the quantro class and constructor. 
#'
#' @param object an object which is inherited from an 
#' \code{eSet} such as an \code{ExpressionSet} or 
#' \code{MethylSet} object. The \code{object} can also be a 
#' data frame or matrix with observations
#' (e.g. probes or genes) on the rows and samples as the 
#' columns.  
#' @param groupFactor a group level factor associated with 
#' each sample or column in the \code{object}. The order of the
#' \code{groupFactor} must match the order of the columns in
#' \code{object}. 
#' @param B number of permutations to assess statistical significance 
#' in a permutation test. Default \code{B}=0. 
#' @param qRange the range of quantiles to consider. Default is 
#' \code{seq(0, 1, length.out = nrow(object))}. 
#' @param useMedianNormalized TRUE/FALSE argument specifying if the 
#' median normalized data should be used or not as input to test for 
#' global differences between distributions. Default is TRUE. 
#' @param verbose TRUE/FALSE argument specifying if verbose messages 
#' should be returned or not. Default is TRUE. 
#' 
#' @return A \code{quantro} S4 class object
#' \item{summary}{Returns a list of three elements 
#' related to a summary of the experiment: 
#' (1) the number of groups (nGroups), 
#' (2) total number of samples (nTotSamples), 
#' (3) number of samples in each group (nSamplesinGroups).}
#' \item{B}{Number of permutations for permutation testing.}
#' \item{anova}{ANOVA to test if the medians of the 
#' distributions (averaged across groups) are different across groups.}
#' \item{quantroStat}{A test statistic which is a ratio of the mean squared 
#' error between groups of distributions to the mean squared error within 
#' groups of distributions (psuedo F-statistic).}
#' \item{quantroStatPerm}{If \code{B} is not equal to 0, then a permutation 
#' test was performed to assess the statistical significance of \code{quantroStat}. 
#' These are the test statistics resulting from the permuted samples.}
#' \item{quantroPvalPerm}{If \code{B} is not equal to 0, then this is the 
#' p-value associated with the proportion of times the test statistics 
#' resulting from the permuted samples were larger than \code{quantroStat}.}
#' 
#' @details 
#' Quantile normalization is one of the most widely used normalization tools 
#' for data analysis in genomics. Although it was originally developed for 
#' gene expression microarrays it is now used across many different 
#' high-throughput applications including RNAseq and ChIPseq. The methodology 
#' relies on the assumption that observed changes in the empirical 
#' distribution of samples are due to unwanted variability. Because the data is 
#' transformed to remove these differences it has the potential to remove 
#' interesting biologically driven global variation. Therefore, applying 
#' quantile normalization, or other global normalization methods
#' that rely on similar assumptions, may not be an appropriate depending 
#' on the type and source of variation. 
#' 
#' This function can be used to test a priori to the data analysis whether 
#' global normalization methods such as quantile normalization should be 
#' applied. The \code{quantro} function uses the raw unprocessed high-throughput 
#' data to test for global differences in the distributions across a set of groups. 
#' 
#' The \code{quantro} function will perform two tests:
#' 
#' 1. An ANOVA to test if the medians of the distributions are different across 
#' groups. Differences across groups could be attributed to unwanted technical 
#' variation (such as batch effects) or real global biological variation. 
#' This is a helpful step for the user to verify if there is some unaccounted
#' technical variation. 
#' 
#' 2. A test for global differences between the distributions across groups.
#' The main output is a test statistic called \code{quantroStat}. This test 
#' statistic is a ratio of two variances and is similar to the idea of ANOVA. 
#' The main idea of the test is to compare the variability of distributions 
#' within the groups to the variability of distributions between the groups. 
#' If the variance between the groups is sufficiently larger than the variance 
#' within the groups, quantile normalization may not be an appropriate 
#' normalization technique depending on the source of variation 
#' (technical or biological variation). As a default, we perform this test on 
#' after a median normalization, but this option may be changed.
#' 
#' To assess the statistical significance of \code{quantroStat}, we use 
#' permutation testing. To perform a permutation test, set \code{B} to the 
#' number of permutations which will create a null distribution.  If the number
#' of samples is large, this number can be a large number such as 1000. This 
#' step can be very slow, but a parallelization has been implemented 
#' throught the \code{foreach} package. Register the number of cores using 
#' the \code{doParallel} package. 
#' 
#' See the vignette for more details. 
#' 
#' @aliases quantro
#' 
#' @docType methods
#' @name show
#' @import Biobase minfi doParallel foreach iterators
#' @importFrom methods show
#' 
#'
#' @examples
#' library(minfi)
#' data(flowSorted)
#' p <- getBeta(flowSorted, offset = 100)
#' pd <- pData(flowSorted)
#' 
#' qtest <- quantro(object = p, groupFactor = pd$CellType)
#' 
#' @rdname quantro
#' @export
quantro <- function(object, groupFactor = NULL, B = 0, qRange = NULL,
                    useMedianNormalized = TRUE, verbose = TRUE)
{ 
    if(is(object, "ExpressionSet")){ object <- exprs(object) }
    if(is(object, "MethylSet")){ object <- getBeta(object, offset = 100) }
  
    if(is.null(groupFactor)){  
        stop("Must provide groupFactor to specify the group 
                level information associated with each sample or 
                or column in object.")
    } else {
        groupFactor <- as.factor(groupFactor)
    }

    if(ncol(object) != length(groupFactor)){
        stop("Number of columns in object does not match 
             length of groupFactor.")
    }

    nT <- ncol(object)
    groupLevels <- levels(groupFactor)
    K <- length(groupLevels)
    nk <- c(table(groupFactor))
    objectMedians <- apply(object, 2, stats::median)
    objectMedians <- round(objectMedians, 7)

    if(length(unique(objectMedians)) == 1){
        anovaFit <- NA
        if(verbose){
            message("[quantro] All median values equal. No ANOVA performed. 
                    No median normalization.")
        }
    } else {
        anovaFit <- anova(stats::lm(objectMedians ~ groupFactor))
        anovaPval <- (anovaFit$`Pr(>F)`[1] < 0.05)
        if(verbose){
            if(anovaPval){ 
                message("[quantro] Average medians of the distributions are 
                        not equal across groups.")
            } else { 
                message("[quantro] Average medians of the distributions are 
                        equal across groups.") 
            }
        }
    }

    if(useMedianNormalized){
        objectMedianNormalized <- sweep(object, 2, objectMedians, FUN = "-")
        objectNorm <- objectMedianNormalized
    } else {
        objectNorm <- object
    }

    if(verbose){ message("[quantro] Calculating the quantro test statistic.") }

    if(is.null(qRange)){
        Fnik = apply(objectNorm, 2, sort) 
    } else {
        Fnik = apply(objectNorm, 2, stats::quantile, probs = qRange, na.rm = TRUE)
    }

    Fndotk = sapply( groupLevels, function(x){ 
        rowMeans(Fnik[, which(groupFactor %in% x), drop = FALSE]) 
    } )
    
    Fndotdot = rowMeans(Fnik)
    
    betweenDiff = colMeans( (Fndotk - replicate(K, Fndotdot))^2 )
    MSb = sum(betweenDiff*nk) / (K - 1)

    withinDiff <- do.call("cbind", lapply(groupLevels, function(x){
        Fnik[, which(groupFactor %in% x) ] - 
            rowMeans(Fnik[, which(groupFactor %in% x), drop = FALSE]) }))

    withinDiff <- colMeans((withinDiff)^2)
    MSe <- sum(withinDiff) / (nT - K)	
    quantroStat <- MSb / MSe

    results <- new("quantro")
    results@summary <- list("nGroups" = K, 
                            "nTotSamples" = nT,
                            "nSamplesinGroups" = nk)
    results@B <- B
    results@anova <- anovaFit
    results@MSbetween <- MSb
    results@MSwithin <- MSe
    results@quantroStat <- quantroStat
    results@quantroStatPerm <- NA
    results@quantroPvalPerm <- NA

    if (B == 0){ message("[quantro] No permutation testing performed. 
                         Use B > 0 for permutation testing.") }
    if (B < 0){ stop("Must pick B greater than or equal to 0.") }
    if (B > 0){
        if(verbose){ message("[quantro] Starting permutation testing.") }
        if(!getDoParRegistered()){ registerDoSEQ() }
        
        workers <- getDoParWorkers()
        backend <- getDoParName() 
        version <- getDoParVersion()

        if (verbose) {
            if (workers == 1) {
                mes <- "[quantro] Using a single core (backend: %s, 
                        version: %s)."
                message(sprintf(mes, backend, version))
            } else {
                mes <- "[quantro] Parallelizing using %s workers/cores 
                        (backend: %s, version: %s)."
                message(sprintf(mes, workers, backend, version))
            }
        }

    one.test <- function(xstar){ 
        FndotkPerm <- sapply(groupLevels, function(x){ 
            rowMeans(Fnik[, which(xstar %in% x) ]) })
        
        betweenDiff <- colMeans( (FndotkPerm - replicate(K, Fndotdot))^2 )
        MSb <- sum(betweenDiff*nk) / (K - 1)
        
        withinDiff <- do.call("cbind", lapply(groupLevels, function(x){ 
            Fnik[, which(xstar %in% x) ] - 
                rowMeans(Fnik[, which(xstar %in% x) ])}))
        withinDiff <- colMeans( (withinDiff)^2 )
        MSe <- sum(withinDiff) / (nT - K)	

        FstatsinglePerm <- MSb / MSe
        return(data.frame(MSb, MSe, FstatsinglePerm))
    }

    xperm <- replicate(B, sample(groupFactor))
    chunksize <- ceiling(B/workers)
    subMat <- NULL
    permResults <- foreach(subMat = iter(xperm, by = "col", 
                                         chunksize = chunksize), 
                           .combine = "c") %dopar% { 
                               apply(subMat, 2, function(x){ one.test(x) }) 
                           }
    permResultDataFrame <- do.call(rbind, permResults)
    quantroPvalPerm <- mean(permResultDataFrame$FstatsinglePerm > quantroStat)

    results@quantroStatPerm <- permResultDataFrame[,3]
    results@quantroPvalPerm <- quantroPvalPerm
    }

    return(results)
}

test_quantro <- function() {
    library(minfi)
    data(flowSorted)
  
    ## Test 1: Using matrix object and groupFactor
    p <- getBeta(flowSorted, offset = 100)
    pd <- pData(flowSorted)
    qtest <- quantro(object = p, groupFactor = pd$CellType)
    checkEqualsNumeric(quantroStat(qtest), 8.807, tolerance=1.0e-4)

    ## Test 2: Using MethylSet object and groupFactor
    qtest <- quantro(object = flowSorted, groupFactor = pd$CellType)
    checkEqualsNumeric(quantroStat(qtest), 8.807, tolerance=1.0e-4)

    ## Test 3: Using MethylSet object, but with parallelization feature
    if(require(doParallel)){
        registerDoParallel(cores = 4)
        
        pd <- pData(flowSorted)
        qtest <- quantro(flowSorted, pd$CellType, B = 100)
        checkEqualsNumeric(quantroStat(qtest), 8.807, tolerance=1.0e-4)
    }
}

#' @name flowSorted
#' @title A subset of FlowSorted.DLPFC.450k data set
#'
#' @description 
#' This is the script used to create a subset of the 
#' FlowSorted.DLPFC.450k data set from Bioconductor. 
#' The purpose is to create an example object for the man
#' pages and vignette in this package. 
#' 
#' @docType data
#' @keywords datasets
#' @format A MethylSet object with 1e4 rows (probes) 
#' and 58 columns (samples).
#' 
#' @rdname flowSorted
#' @export
#' 
library(FlowSorted.DLPFC.450k)
library(minfi)
data(FlowSorted.DLPFC.450k)

Mset <- preprocessRaw(FlowSorted.DLPFC.450k)
pd <- pData(Mset)
meth <- getMeth(Mset)
unmeth <- getUnmeth(Mset)

# Subset probes to speed up computation time for vignette 
# and man pages
set.seed(123)
subRows <- sample(seq_len(nrow(meth)), size = 1e4)
meth <- meth[subRows,]
unmeth <- unmeth[subRows,]

# Create MethylSet object
flowSorted <- MethylSet(Meth = meth, Unmeth = unmeth, 
                        phenoData = AnnotatedDataFrame(pd))

# Save MethylSet object to use in vignette and man pages
save(flowSorted, file = "data/flowSorted.RData", compress='xz') 
# need to compress to save space in bioC pkg

## ----echo=FALSE, results="hide", message=FALSE--------------------------------
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----install-quantro----------------------------------------------------------
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("quantro")

## ----lib-load, message=FALSE--------------------------------------------------
library(quantro)

## ----data-load, message=FALSE-------------------------------------------------
library(minfi)
data(flowSorted)
p <- getBeta(flowSorted, offset = 100)
pd <- pData(flowSorted)
dim(p)
head(pd)

## ----plot-distributions-density, fig.height=5, fig.width=6--------------------
matdensity(p, groupFactor = pd$CellType, xlab = " ", ylab = "density",
           main = "Beta Values", brewer.n = 8, brewer.name = "Dark2")
legend('top', c("NeuN_neg", "NeuN_pos"), col = c(1, 2), lty = 1, lwd = 3)

## ----plot-distributions-boxplot, fig.height=5, fig.width=6--------------------
matboxplot(p, groupFactor = pd$CellType, xaxt = "n", main = "Beta Values")

## ----calculate-quantro1-------------------------------------------------------
qtest <- quantro(object = p, groupFactor = pd$CellType)
qtest

## ----quantro-summary----------------------------------------------------------
summary(qtest)

## ----quantro-medians----------------------------------------------------------
anova(qtest)

## ----quantro-quantroStat------------------------------------------------------
quantroStat(qtest)

## ----flowSorted-fullEx--------------------------------------------------------
is(flowSorted, "MethylSet")
qtest <- quantro(flowSorted, groupFactor = pData(flowSorted)$CellType)
qtest 

## ----quantro-parallel---------------------------------------------------------
library(doParallel)
registerDoParallel(cores=1)
qtestPerm <- quantro(p, groupFactor = pd$CellType, B = 1000)
qtestPerm

## ----quantro-plot, fig.height=5, fig.width=6----------------------------------
quantroPlot(qtestPerm)

## ----sessionInfo, results='markup'--------------------------------------------
sessionInfo()

#' @rdname summary
#' @export
setGeneric("summary", function(object) standardGeneric("summary"))

#' @rdname anova
#' @export
setGeneric("anova", function(object) standardGeneric("anova"))

#' @rdname MSbetween
#' @export
setGeneric("MSbetween", function(object) standardGeneric("MSbetween"))

#' @rdname MSwithin
#' @export
setGeneric("MSwithin", function(object) standardGeneric("MSwithin"))

#' @rdname quantroStat
#' @export
setGeneric("quantroStat", function(object) standardGeneric("quantroStat"))

#' @rdname quantroStatPerm
#' @export
setGeneric("quantroStatPerm", function(object) 
    standardGeneric("quantroStatPerm"))

#' @rdname quantroPvalPerm
#' @export
setGeneric("quantroPvalPerm", function(object) 
    standardGeneric("quantroPvalPerm"))

---
title: "The quantro user's guide"
author:
- name: Stephanie C. Hicks
  affiliation: Johns Hopkins Bloomberg School of Public Health
- name: Rafael Irizarry
  affiliation: Dana-Farber Cancer Institute
output:
  BiocStyle::html_document:
    toc_float: true
package: quantro
abstract: |
 A test for when to use global normalization methods, such as quantile normalization.
bibliography: quantro.bib
vignette: |
  %\VignetteIndexEntry{The quantro user's guide}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, echo=FALSE, results="hide", message=FALSE}
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```

# Introduction

Multi-sample normalization techniques such as quantile normalization 
@Bolstad2003, @Irizarry2003 have become a standard and essential part of 
analysis pipelines for high-throughput data. Although it was originally 
developed for gene expression microarrays, it is now used across many 
different high-throughput applications including genotyping arrays, DNA 
Methylation, RNA Sequencing (RNA-Seq) and Chromatin Immunoprecipitation 
Sequencing (ChIP-Seq). These techniques transform the original raw data to 
remove unwanted technical variation. However, quantile normalization and other 
global normalization methods rely on assumptions about the data generation 
process that are not appropriate in some context. Until now, it has been left 
to the researcher to check for the appropriateness of these assumptions. 

Quantile normalization assumes that the statistical distribution of each 
sample is the same. Normalization is achieved by forcing the observed 
distributions to be the same and the average distribution, obtained by taking 
the average of each quantile across samples, is used as the reference. This 
method has worked very well in practice but note that when the assumptions are 
not met, global changes in distribution, that may be of biological interest, 
will be wiped out and features that are not different across samples can be 
artificially induced.  These types of assumptions are justified in many 
biomedical applications, for example in gene expression studies in which only 
a minority of genes are expected to be differentially expressed. However, if, 
for example, a substantially higher percentage of genes are expected to be 
expressed in only one group of samples, it may not be appropriate to use 
global adjustment methods. 

The `quantro` R-package can be used to test for global differences 
between groups of distributions which asses whether global normalization 
methods such as quantile normalization should be applied. Our method uses 
the raw unprocessed high-throughput data to test for global differences in 
the distributions across a set of groups. The main function `quantro()` 
will perform two tests: 

1. An ANOVA to test if the medians of the distributions are different 
across groups. Differences across groups could be attributed to unwanted 
technical variation (such as batch effects) or real global biological 
variation. This is a helpful step for the user to verify if there is any 
technical variation unaccounted for. 

2. A test for global differences between the distributions across groups 
which returns a test statistic called `quantroStat`. This test 
statistic is a ratio of two variances and is similar to the idea of ANOVA. 
The main idea is to compare the variability of distributions within groups 
relative to between groups. If the variability between groups is sufficiently 
larger than the variability within groups, then this suggests global 
adjustment methods may not be appropriate. As a default, we perform this test 
on the median normalized data, but the user may change this option. 

# Getting Started

## Installation

The R-package **quantro** can be [installed from the Bioconductor](http://www.bioconductor.org/packages/release/bioc/html/quantro.html)
```{r install-quantro}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("quantro")
```

## Load the package in R

After installation, the package can be loaded into R.
```{r lib-load, message=FALSE}
library(quantro)
```



# Data

## `flowSorted` Data Example

To explore how to use `quantro()`, we use the 
`FlowSorted.DLPFC.450k` data package in Bioconductor 
@JaffeFlowSorted. 
The data in this package originally came from @Guintivano2013. 
This data set in `FlowSorted.DLPFC.450k` contains raw data objects of 58 
Illumina 450K DNA methylation microarrays, formatted as `RGset` 
objects. The samples represent two different cellular populations of brain 
tissues on the same 29 individuals extracted using flow sorting.  For more 
information on this data set, please see the `FlowSorted.DLPFC.450k` User's 
Guide.For the purposes of this vignette, a MethylSet object from the 
`minfi` Bioconductor package @Aryee2014 was created which is 
a subset of the rows from the original `FlowSorted.DLPFC.450k` data 
set. This `MethylSet` object is found in the /data folder and the script to 
create the object is found in /inst.  

Here we will explore the distributions of these two cellular populations of 
brain tissue (`NeuN_pos` and `NeuN_neg`) and then test if there 
are global differences in the distributions across groups. First, load the 
MethylSet object (`flowSorted`) and compute the Beta values using 
the function `getBeta()` in the `minfi` Bioconductor package. 
We use an offset of 100 as this is the default used by Illumina. 

```{r data-load, message=FALSE}
library(minfi)
data(flowSorted)
p <- getBeta(flowSorted, offset = 100)
pd <- pData(flowSorted)
dim(p)
head(pd)
```

## Plot distributions

`quantro` contains two functions to view the distributions of the 
samples of interest: `matdensity()` and `matboxplot()`. The function 
`matdensity()` computes the density for each sample (columns) and 
uses the `matplot()` function to plot all the densities. 
`matboxplot()` orders and colors the samples by a group level variable. 
These two functions use the `RColorBrewer` package and the brewer 
palettes can be changed using the arguments `brewer.n` and 
`brewer.name`. 

The distributions of the two groups of cellular populations are shown here. 
The `NeuN_neg` samples are colored in green and the `NeuN_pos` are 
colored in red.  

```{r plot-distributions-density, fig.height=5, fig.width=6}
matdensity(p, groupFactor = pd$CellType, xlab = " ", ylab = "density",
           main = "Beta Values", brewer.n = 8, brewer.name = "Dark2")
legend('top', c("NeuN_neg", "NeuN_pos"), col = c(1, 2), lty = 1, lwd = 3)
```

```{r plot-distributions-boxplot, fig.height=5, fig.width=6}
matboxplot(p, groupFactor = pd$CellType, xaxt = "n", main = "Beta Values")
```


# Using the `quantro()` function

## Input for `quantro()`

The `quantro()` function must have two objects as input: 

1. An `object` which is a data frame or matrix with observations 
(e.g. probes or genes) on the rows and samples as the columns. 

2. A `groupFactor` which represents the group level information 
about each sample. For example if the samples represent tumor and normal 
samples, provide `quantro()` with a factor representing which columns 
in the `object` are normal and tumor samples.

## Running `quantro()`

In this example, the groups we are interested in comparing are contained in 
the `CellType` column in the `pd` dataset. To run the 
`quantro()` function, input the data object and the object containing 
the phenotypic data. Here we use the `flowSorted` data set as an 
example. 

```{r calculate-quantro1}
qtest <- quantro(object = p, groupFactor = pd$CellType)
qtest
```

The details related to the experiment can be extracted using the 
`summary` accessor function:

```{r quantro-summary}
summary(qtest)
```

To asssess if the medians of the distributions different across groups, 
we perform an ANOVA on the medians from the samples. Those results can be 
found using `anova`:

```{r quantro-medians}
anova(qtest)
```

The full output can be seen The test statistic produced from 
`quantro()` testing for global differences between distributions 
is given by `quantroStat`: 

```{r quantro-quantroStat}
quantroStat(qtest)
```


## eSets

`quantro()` also can accept objects that inherit `eSets`
such as an `ExpressionSet` or `MethylSet`. The 
`groupFactor` must still be provided.  

```{r flowSorted-fullEx}
is(flowSorted, "MethylSet")
qtest <- quantro(flowSorted, groupFactor = pData(flowSorted)$CellType)
qtest 
```

## Output from `quantro()`


Elements in the S4 object from `quantro()` include: 

Element | Description
--------|------------
`summary` | A list that contains (1) number of groups (`nGroups`), (2) total number of samples (`nTotSamples`) (3) number of samples in each group (`nSamplesinGroups`)
`anova` | ANOVA to test if the average medians of the distributions are different across groups
`MSbetween` | mean squared error between groups
`MSwithin` | mean squared error within groups
`quantroStat` | test statistic which is a ratio of the mean squared error between groups of distributions to the mean squared error within groups of distributions
`quantroStatPerm` | If `B` is not equal to 0, then a permutation test was performed to assess the statistical significance of `quantroStat`. These are the test statistics resulting from the permuted samples
`quantroPvalPerm` | If `B` is not equal to 0, then this is the $p$-value associated with the proportion of times the test statistics (`quantroStatPerm`) resulting from the permuted samples were larger than `quantroStat`


# Assessing the statistical significance

To assess statistical significance of the test statistic, we use 
permutation testing.  We use the `foreach` package which distribute 
the computations across multiple cross in a single machine or across 
multiple machines in a cluster. The user must pick how many permutations 
to perform where `B` is the number of permutations. At each 
permutation of the samples, a test statistic is calculated. The proportion 
of test statistics (`quantroStatPerm`) that are larger than the 
`quantroStat` is reported as the `quantroPvalPerm`. To use 
the `foreach` package, we first register a backend, in this case a 
machine with 1 cores. 

```{r quantro-parallel}
library(doParallel)
registerDoParallel(cores=1)
qtestPerm <- quantro(p, groupFactor = pd$CellType, B = 1000)
qtestPerm
```


# Visualizing the statistical significance from permutation tests

If permutation testing was used (i.e. specifying `B` $>$ 0), then 
there is a second function in the package called `quantroPlot()` 
which will plot the test statistics of the permuted samples. The plot is 
a histogram of the null test statistics `quantroStatPerm` from 
`quantro()` and the red line is the observed test statistic 
`quantroStat` from `quantro()`. 

```{r quantro-plot, fig.height=5, fig.width=6}
quantroPlot(qtestPerm)
```

Additional options in the `quantroPlot()` function include:

Element | Description
--------|-------------
xLab | the x-axis label
yLab | the y-axis label
mainLab | title of the histogram
binWidth | change the binwidth


# SessionInfo

```{r sessionInfo, results='markup'}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{quantro-class}
\alias{quantro-class}
\title{quantro}
\description{
This is the S4 class quantro container
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{quantroStat}
\alias{quantroStat}
\alias{quantroStat.quantro}
\alias{quantroStat,quantro-method}
\title{Accessors for the 'quantroStat' slot of a quantro object.}
\usage{
quantroStat(object)

\S4method{quantroStat}{quantro}(object)

\S4method{quantroStat}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}
}
\description{
Accessors for the 'quantroStat' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
quantroStat(qtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{quantroStatPerm}
\alias{quantroStatPerm}
\alias{quantroStatPerm.quantro}
\alias{quantroStatPerm,quantro-method}
\title{Accessors for the 'quantroStatPerm' slot of a quantro object.}
\usage{
quantroStatPerm(object)

\S4method{quantroStatPerm}{quantro}(object)

\S4method{quantroStatPerm}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}
}
\description{
Accessors for the 'quantroStatPerm' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
quantroStatPerm(qtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{matboxplot}
\alias{matboxplot}
\title{Box plots of columns in a matrix}
\usage{
matboxplot(
  object,
  groupFactor,
  las = 3,
  brewer.n = 8,
  brewer.name = "Dark2",
  ...
)
}
\arguments{
\item{object}{object an object which is inherited from an 
\code{eSet} such as an \code{ExpressionSet} or 
\code{MethylSet} object. The \code{object} can also be a 
data frame or matrix with observations
(e.g. probes or genes) on the rows and samples as the 
columns.}

\item{groupFactor}{a factor variable representing which 
group each column in \code{object} belongs to. It is important 
that values in \code{groupFactor} be in the same 
order as the columns in \code{object}.}

\item{las}{a numeric in (0, 1, 2, 3) to orient the axis labels. 
Default is 3 (always vertical).}

\item{brewer.n}{the number of colors in the palette from the RColorBrewer 
package. Default is 8.}

\item{brewer.name}{the name of the palette from the RColorBrewer package. 
Default is "Dark2".}

\item{...}{other arguments that can be passed to the 
code{boxplot} function.}
}
\value{
A box plot for each column in \code{object}
}
\description{
Box plots of the columns of a matrix, but 
the columns are ordered and colored by a group-level 
variable
}
\examples{
library(minfi)
data(flowSorted)

p <- getBeta(flowSorted, offset = 100)
pd <- pData(flowSorted)
matboxplot(object = p, groupFactor = pd$CellType)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{anova}
\alias{anova}
\alias{anova.quantro}
\alias{anova,quantro-method}
\title{Accessors for the 'anova' slot of a quantro object.}
\usage{
anova(object, ...)

\S4method{anova}{quantro}(object)

\S4method{anova}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}

\item{...}{other}
}
\description{
Accessors for the 'anova' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
anova(qtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{methods}
\name{show}
\alias{show}
\alias{quantro}
\title{quantro}
\usage{
quantro(
  object,
  groupFactor = NULL,
  B = 0,
  qRange = NULL,
  useMedianNormalized = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{object}{an object which is inherited from an 
\code{eSet} such as an \code{ExpressionSet} or 
\code{MethylSet} object. The \code{object} can also be a 
data frame or matrix with observations
(e.g. probes or genes) on the rows and samples as the 
columns.}

\item{groupFactor}{a group level factor associated with 
each sample or column in the \code{object}. The order of the
\code{groupFactor} must match the order of the columns in
\code{object}.}

\item{B}{number of permutations to assess statistical significance 
in a permutation test. Default \code{B}=0.}

\item{qRange}{the range of quantiles to consider. Default is 
\code{seq(0, 1, length.out = nrow(object))}.}

\item{useMedianNormalized}{TRUE/FALSE argument specifying if the 
median normalized data should be used or not as input to test for 
global differences between distributions. Default is TRUE.}

\item{verbose}{TRUE/FALSE argument specifying if verbose messages 
should be returned or not. Default is TRUE.}
}
\value{
A \code{quantro} S4 class object
\item{summary}{Returns a list of three elements 
related to a summary of the experiment: 
(1) the number of groups (nGroups), 
(2) total number of samples (nTotSamples), 
(3) number of samples in each group (nSamplesinGroups).}
\item{B}{Number of permutations for permutation testing.}
\item{anova}{ANOVA to test if the medians of the 
distributions (averaged across groups) are different across groups.}
\item{quantroStat}{A test statistic which is a ratio of the mean squared 
error between groups of distributions to the mean squared error within 
groups of distributions (psuedo F-statistic).}
\item{quantroStatPerm}{If \code{B} is not equal to 0, then a permutation 
test was performed to assess the statistical significance of \code{quantroStat}. 
These are the test statistics resulting from the permuted samples.}
\item{quantroPvalPerm}{If \code{B} is not equal to 0, then this is the 
p-value associated with the proportion of times the test statistics 
resulting from the permuted samples were larger than \code{quantroStat}.}
}
\description{
This is a function that tests for global differences between 
groups of distributions which asses whether global normalization 
methods such as quantile normalization should be applied. 
This function defines the quantro class and constructor.
}
\details{
Quantile normalization is one of the most widely used normalization tools 
for data analysis in genomics. Although it was originally developed for 
gene expression microarrays it is now used across many different 
high-throughput applications including RNAseq and ChIPseq. The methodology 
relies on the assumption that observed changes in the empirical 
distribution of samples are due to unwanted variability. Because the data is 
transformed to remove these differences it has the potential to remove 
interesting biologically driven global variation. Therefore, applying 
quantile normalization, or other global normalization methods
that rely on similar assumptions, may not be an appropriate depending 
on the type and source of variation. 

This function can be used to test a priori to the data analysis whether 
global normalization methods such as quantile normalization should be 
applied. The \code{quantro} function uses the raw unprocessed high-throughput 
data to test for global differences in the distributions across a set of groups. 

The \code{quantro} function will perform two tests:

1. An ANOVA to test if the medians of the distributions are different across 
groups. Differences across groups could be attributed to unwanted technical 
variation (such as batch effects) or real global biological variation. 
This is a helpful step for the user to verify if there is some unaccounted
technical variation. 

2. A test for global differences between the distributions across groups.
The main output is a test statistic called \code{quantroStat}. This test 
statistic is a ratio of two variances and is similar to the idea of ANOVA. 
The main idea of the test is to compare the variability of distributions 
within the groups to the variability of distributions between the groups. 
If the variance between the groups is sufficiently larger than the variance 
within the groups, quantile normalization may not be an appropriate 
normalization technique depending on the source of variation 
(technical or biological variation). As a default, we perform this test on 
after a median normalization, but this option may be changed.

To assess the statistical significance of \code{quantroStat}, we use 
permutation testing. To perform a permutation test, set \code{B} to the 
number of permutations which will create a null distribution.  If the number
of samples is large, this number can be a large number such as 1000. This 
step can be very slow, but a parallelization has been implemented 
throught the \code{foreach} package. Register the number of cores using 
the \code{doParallel} package. 

See the vignette for more details.
}
\examples{
library(minfi)
data(flowSorted)
p <- getBeta(flowSorted, offset = 100)
pd <- pData(flowSorted)

qtest <- quantro(object = p, groupFactor = pd$CellType)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{MSwithin}
\alias{MSwithin}
\alias{MSwithin.quantro}
\alias{MSwithin,quantro-method}
\title{Accessors for the 'MSwithin' slot of a quantro object.}
\usage{
MSwithin(object)

\S4method{MSwithin}{quantro}(object)

\S4method{MSwithin}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}
}
\description{
Accessors for the 'MSwithin' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
MSwithin(qtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{summary}
\alias{summary}
\alias{summary.quantro}
\alias{summary,quantro-method}
\title{Accessors for the 'summary' slot of a quantro object.}
\usage{
summary(object, ...)

\S4method{summary}{quantro}(object)

\S4method{summary}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}

\item{...}{other}
}
\description{
Accessors for the 'summary' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
summary(qtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{MSbetween}
\alias{MSbetween}
\alias{MSbetween.quantro}
\alias{MSbetween,quantro-method}
\title{Accessors for the 'MSbetween' slot of a quantro object.}
\usage{
MSbetween(object)

\S4method{MSbetween}{quantro}(object)

\S4method{MSbetween}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}
}
\description{
Accessors for the 'MSbetween' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
MSbetween(qtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{quantroPlot}
\alias{quantroPlot}
\title{Plot results from \code{quantro} function.}
\usage{
quantroPlot(
  object,
  savePlot = FALSE,
  xLab = NULL,
  yLab = NULL,
  mainLab = NULL,
  binWidth = NULL
)
}
\arguments{
\item{object}{a quantro object from \code{quantro}}

\item{savePlot}{a TRUE/FALSE object argument
determining if the plot will be saved for further use or
immediately displayed on the screen.}

\item{xLab}{label for x-axis}

\item{yLab}{label for y-axis}

\item{mainLab}{title of plot}

\item{binWidth}{binwidth or histogram. Default is the stat_bin default.}
}
\value{
A histogram will be plotted containing the null 
test statistics when using boostrapped samples. 
The red line is the observed test statistic
\code{quantroStat} from \code{quantro()}.
}
\description{
This function plots the histogram of the
null test statistics from permutation test in the \code{quantro}
function.
}
\examples{
\donttest{
library(minfi)
data(flowSorted)
p <- getBeta(flowSorted, offset = 100)
pd <- pData(flowSorted)

library(doParallel)
registerDoParallel(cores=4)
qtest <- quantro(p, pd$CellType, B = 100)
quantroPlot(qtest)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flowSorted.R
\docType{data}
\name{flowSorted}
\alias{flowSorted}
\title{A subset of FlowSorted.DLPFC.450k data set}
\format{A MethylSet object with 1e4 rows (probes) 
and 58 columns (samples).}
\description{
This is the script used to create a subset of the 
FlowSorted.DLPFC.450k data set from Bioconductor. 
The purpose is to create an example object for the man
pages and vignette in this package. 

The object was created using the script in /inst and
located in the /data folder.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{matdensity}
\alias{matdensity}
\title{Density plots of columns in a matrix}
\usage{
matdensity(
  object,
  groupFactor = NULL,
  type = "l",
  lty = 1,
  brewer.n = 8,
  brewer.name = "Dark2",
  ...
)
}
\arguments{
\item{object}{object an object which is inherited from an 
\code{eSet} such as an \code{ExpressionSet} or 
\code{MethylSet} object. The \code{object} can also be a 
data frame or matrix with observations
(e.g. probes or genes) on the rows and samples as the 
columns.}

\item{groupFactor}{an optional factor variable representing which 
group each column in \code{object} belongs to. It is important 
that values in \code{groupFactor} be in the same 
order as the columns in \code{object}.}

\item{type}{the type of lines to plot. Default type is line ("l").}

\item{lty}{the line type. Default is the solid line.}

\item{brewer.n}{the number of colors in the palette from the RColorBrewer 
package. Default is 8.}

\item{brewer.name}{the name of the palette from the RColorBrewer package. 
Default is "Dark2".}

\item{...}{other arguments that can be passed to the 
code{matplot} function.}
}
\value{
A density plot for each column in \code{object}
}
\description{
Plots the density of the columns of a matrix
}
\examples{
library(minfi)
data(flowSorted)
p <- getBeta(flowSorted, offset = 100)
pd <- pData(flowSorted)
matdensity(object = p, groupFactor = pd$CellType, xlab = "beta values",
ylab = "density")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{quantroPvalPerm}
\alias{quantroPvalPerm}
\alias{quantroPvalPerm.quantro}
\alias{quantroPvalPerm,quantro-method}
\title{Accessors for the 'quantroPvalPerm' slot of a quantro object.}
\usage{
quantroPvalPerm(object)

\S4method{quantroPvalPerm}{quantro}(object)

\S4method{quantroPvalPerm}{quantro}(object)
}
\arguments{
\item{object}{a \code{quantro} object}
}
\description{
Accessors for the 'quantroPvalPerm' slot of a quantro object.
}
\examples{
library(minfi)
data(flowSorted)
pd <- pData(flowSorted)
qtest <- quantro(flowSorted, groupFactor = pd$CellType)
quantroPvalPerm(qtest)
}
